@@ -1,7 +1,21 @@
 # -*- coding: utf-8 -*-
 # copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
-# contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
-# license: General Public License version 2 - http://www.gnu.org/licenses
+# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
+#
+# This file is part of logilab-common.
+#
+# logilab-common is free software: you can redistribute it and/or modify it under
+# the terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation, either version 2.1 of the License, or (at your option) any
+# later version.
+#
+# logilab-common is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
+# details.
+#
+# You should have received a copy of the GNU Lesser General Public License along
+# with logilab-common.  If not, see <http://www.gnu.org/licenses/>.
 """Run tests.
 
 This will find all modules whose name match a given prefix in the test
@@ -46,8 +60,14 @@ import warnings
 from compiler.consts import CO_GENERATOR
 from ConfigParser import ConfigParser
 from itertools import dropwhile
-from functools import wraps
-
+try:
+    from functools import wraps
+except ImportError:
+    def wraps(wrapped):
+        def proxy(callable):
+            callable.__name__ = wrapped.__name__
+            return callable
+        return proxy
 try:
     from test import test_support
 except ImportError:
@@ -58,7 +78,7 @@ except ImportError:
     test_support = TestSupport()
 
 # pylint: disable-msg=W0622
-from logilab.common.compat import set, enumerate, any, sorted
+from logilab.common.compat import set, enumerate, any, sorted, InheritableSet
 # pylint: enable-msg=W0622
 from logilab.common.modutils import load_module_from_name
 from logilab.common.debugger import Debugger, colorize_source
@@ -197,8 +217,8 @@ def run_test(test, verbose, runner=None, capture=0):
         if runner is None:
             runner = SkipAwareTextTestRunner(capture=capture) # verbosity=0)
         return runner.run(suite)
-    except KeyboardInterrupt, v:
-        raise KeyboardInterrupt, v, sys.exc_info()[2]
+    except KeyboardInterrupt:
+        raise
     except:
         # raise
         type, value = sys.exc_info()[:2]
@@ -488,13 +508,10 @@ class SkipAwareTextTestRunner(unittest.TextTestRunner):
         if self.options is not None:
             tags_pattern = getattr(self.options, 'tags_pattern', None)
             if tags_pattern is not None:
-                tags = getattr(test, 'tags', None)
-                if tags is not None:
-                    return tags.match(tags_pattern)
-                if isinstance(test, types.MethodType):
-                    tags = getattr(test.im_class, 'tags', Tags())
-                    return tags.match(tags_pattern)
-                return False
+                tags = getattr(test, 'tags', Tags())
+                if tags.inherit and isinstance(test, types.MethodType):
+                    tags = tags | getattr(test.im_class, 'tags', Tags())
+                return tags.match(tags_pattern)
         return True # no pattern
 
     def _makeResult(self):
@@ -817,17 +834,14 @@ Examples:
             try:
                 restartfile = open(FILE_RESTART, 'r')
                 try:
-                    try:
-                        succeededtests = list(elem.rstrip('\n\r') for elem in
-                            restartfile.readlines())
-                        removeSucceededTests(self.test, succeededtests)
-                    except Exception, e:
-                        raise e
+                    succeededtests = list(elem.rstrip('\n\r') for elem in
+                                          restartfile.readlines())
+                    removeSucceededTests(self.test, succeededtests)
                 finally:
                     restartfile.close()
-            except Exception ,e:
-                raise "Error while reading \
-succeeded tests into", osp.join(os.getcwd(),FILE_RESTART)
+            except Exception, ex:
+                raise Exception("Error while reading succeeded tests into %s: %s"
+                                % (osp.join(os.getcwd(), FILE_RESTART), ex))
 
         result = self.testRunner.run(self.test)
         # help garbage collection: we want TestSuite, which hold refs to every
@@ -981,12 +995,30 @@ class InnerTest(tuple):
         instance.name = name
         return instance
 
+class Tags(InheritableSet): # 2.4 compat
+    """A set of tag able validate an expression"""
+
+    def __init__(self, *tags, **kwargs):
+        self.inherit = kwargs.pop('inherit', True)
+        if kwargs:
+           raise TypeError("%s are an invalid keyword argument for this function" % kwargs.keys())
+
+        if len(tags) == 1 and not isinstance(tags[0], basestring):
+            tags = tags[0]
+        super(Tags, self).__init__(tags, **kwargs)
+
+    def __getitem__(self, key):
+        return key in self
+
+    def match(self, exp):
+        return eval(exp, {}, self)
 
 class TestCase(unittest.TestCase):
-    """unittest.TestCase with some additional methods"""
+    """A unittest.TestCase extension with some additional methods."""
 
     capture = False
     pdbclass = Debugger
+    tags = Tags()
 
     def __init__(self, methodName='runTest'):
         super(TestCase, self).__init__(methodName)
@@ -1004,6 +1036,8 @@ class TestCase(unittest.TestCase):
         self._current_test_descr = None
         self._options_ = None
 
+    @classproperty
+    @cached
     def datadir(cls): # pylint: disable-msg=E0213
         """helper attribute holding the standard test's data directory
 
@@ -1013,12 +1047,11 @@ class TestCase(unittest.TestCase):
         return osp.join(osp.dirname(osp.abspath(mod.__file__)), 'data')
     # cache it (use a class method to cache on class since TestCase is
     # instantiated for each test run)
-    datadir = classproperty(cached(datadir))
 
+    @classmethod
     def datapath(cls, *fname):
         """joins the object's datadir and `fname`"""
         return osp.join(cls.datadir, *fname)
-    datapath = classmethod(datapath)
 
     def set_description(self, descr):
         """sets the current test's description.
@@ -1140,19 +1173,16 @@ class TestCase(unittest.TestCase):
                     try:
                         restartfile = open(FILE_RESTART, 'a')
                         try:
-                            try:
-                                descr = '.'.join((self.__class__.__module__,
-                                    self.__class__.__name__,
-                                    self._testMethodName))
-                                restartfile.write(descr+os.linesep)
-                            except Exception, e:
-                                raise e
+                            descr = '.'.join((self.__class__.__module__,
+                                              self.__class__.__name__,
+                                              self._testMethodName))
+                            restartfile.write(descr+os.linesep)
                         finally:
                             restartfile.close()
-                    except Exception, e:
+                    except Exception, ex:
                         print >> sys.__stderr__, "Error while saving \
 succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
-                        raise e
+                        raise ex
                 result.addSuccess(self)
         finally:
             # if result.cvg:
@@ -1239,19 +1269,32 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
         msg = msg or 'test was skipped'
         raise InnerTestSkipped(msg)
 
-    def assertIn(self, object, set):
-        """assert <object> are in <set>"""
-        self.assert_(object in set, "%s not in %s" % (object, set))
+    def assertIn(self, object, set, msg=None):
+        """assert <object> is in <set>
 
-    def assertNotIn(self, object, set):
-        """assert <object> are not in <set>"""
-        self.assert_(object not in set, "%s in %s" % (object, set))
+        :param object: a Python Object
+        :param set: a Python Container
+        :param msg: custom message (String) in case of failure
+        """
+        self.assert_(object in set, msg or "%s not in %s" % (object, set))
 
-    def assertDictEquals(self, dict1, dict2):
+    def assertNotIn(self, object, set, msg=None):
+        """assert <object> is not in <set>
+
+        :param object: a Python Object
+        :param set: the Python container to contain <object>
+        :param msg: custom message (String) in case of failure
+        """
+        self.assert_(object not in set, msg or "%s in %s" % (object, set))
+
+    def assertDictEquals(self, dict1, dict2, msg=None):
         """compares two dicts
 
         If the two dict differ, the first difference is shown in the error
         message
+        :param dict1: a Python Dictionary
+        :param dict2: a Python Dictionary
+        :param msg: custom message (String) in case of failure
         """
         dict1 = dict(dict1)
         msgs = []
@@ -1265,14 +1308,19 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
                 msgs.append('missing %r key' % key)
         if dict1:
             msgs.append('dict2 is lacking %r' % dict1)
-        if msgs:
+        if msg:
+            self.failureException(msg)
+        elif msgs:
             self.fail('\n'.join(msgs))
     assertDictEqual = assertDictEquals
 
-
-
     def assertUnorderedIterableEquals(self, got, expected, msg=None):
-        """compares two iterable and shows difference between both"""
+        """compares two iterable and shows difference between both
+
+        :param got: the unordered Iterable that we found
+        :param expected: the expected unordered Iterable
+        :param msg: custom message (String) in case of failure
+        """
         got, expected = list(got), list(expected)
         self.assertSetEqual(set(got), set(expected), msg)
         if len(got) != len(expected):
@@ -1298,6 +1346,15 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
     assertUnordIterEquals = assertUnordIterEqual = assertUnorderedIterableEqual
 
     def assertSetEquals(self,got,expected, msg=None):
+        """compares two sets and shows difference between both
+
+        Don't use it for iterables other than sets.
+
+        :param got: the Set that we found
+        :param expected: the second Set to be compared to the first one
+        :param msg: custom message (String) in case of failure
+        """
+
         if not(isinstance(got, set) and isinstance(expected, set)):
             warnings.warn("the assertSetEquals function if now intended for set only."\
                           "use assertUnorderedIterableEquals instead.",
@@ -1321,6 +1378,10 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
 
         If the two list differ, the first difference is shown in the error
         message
+
+        :param list_1: a Python List
+        :param list_2: a second Python List
+        :param msg: custom message (String) in case of failure
         """
         _l1 = list_1[:]
         for i, value in enumerate(list_2):
@@ -1342,21 +1403,29 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
             self.fail(msg)
     assertListEqual = assertListEquals
 
-    def assertLinesEquals(self, list_1, list_2, msg=None, striplines=False):
-        """assert list of lines are equal"""
-        lines1 = list_1.splitlines()
+    def assertLinesEquals(self, string1, string2, msg=None, striplines=False):
+        """compare two strings and assert that the text lines of the strings
+        are equal.
+
+        :param string1: a String
+        :param string2: a String
+        :param msg: custom message (String) in case of failure
+        :param striplines: Boolean to trigger line stripping before comparing
+        """
+        lines1 = string1.splitlines()
+        lines2 = string2.splitlines()
         if striplines:
             lines1 = [l.strip() for l in lines1]
-        lines2 = list_2.splitlines()
-        if striplines:
             lines2 = [l.strip() for l in lines2]
         self.assertListEquals(lines1, lines2, msg)
     assertLineEqual = assertLinesEquals
 
     def assertXMLWellFormed(self, stream, msg=None, context=2):
         """asserts the XML stream is well-formed (no DTD conformance check)
-        :context: number of context lines in standard msg. all data if negativ
-                  only available with element tree
+
+        :param context: number of context lines in standard message
+                        (show all data if negative).
+                        Only available with element tree
         """
         try:
             from xml.etree.ElementTree import parse
@@ -1377,24 +1446,26 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
 
     def assertXMLStringWellFormed(self, xml_string, msg=None, context=2):
         """asserts the XML string is well-formed (no DTD conformance check)
-        :context: number of context lines in standard msg. all data if negativ
-                  only available with element tree
+
+        :param context: number of context lines in standard message
+                        (show all data if negative).
+                        Only available with element tree
         """
         try:
             from xml.etree.ElementTree import fromstring
-            self._assertETXMLWellFormed(xml_string, fromstring, msg)
         except ImportError:
-            raise
-            stream = StringIO(xml_string)
-            self.assertXMLWellFormed(stream, msg)
+            from elementtree.ElementTree import fromstring
+        self._assertETXMLWellFormed(xml_string, fromstring, msg)
 
     def _assertETXMLWellFormed(self, data, parse, msg=None, context=2):
         """internal function used by /assertXML(String)?WellFormed/ functions
-        :data: xml_data
-        :parse: appropriate parser function for this data
-        :msg: error message
-        :context: number of context lines in standard msg. all data if negativ
-                  only available with element tree
+
+        :param data: xml_data
+        :param parse: appropriate parser function for this data
+        :param msg: error message
+        :param context: number of context lines in standard message
+                        (show all data if negative).
+                        Only available with element tree
         """
         from xml.parsers.expat import ExpatError
         try:
@@ -1477,8 +1548,15 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
                 self.fail('\n'.join(['%s\n'%msg_prefix]+read + list(result)))
 
     def assertTextEquals(self, text1, text2, junk=None,
-            msg_prefix='Text differ'):
-        """compare two multiline strings (using difflib and splitlines())"""
+            msg_prefix='Text differ', striplines=False):
+        """compare two multiline strings (using difflib and splitlines())
+        
+        :param text1: a Python BaseString
+        :param text2: a second Python Basestring
+        :param junk: List of Caracters
+        :param msg_prefix: String (message prefix)
+        :param striplines: Boolean to trigger line stripping before comparing
+        """
         msg = []
         if not isinstance(text1, basestring):
             msg.append('text1 is not a string (%s)'%(type(text1)))
@@ -1486,8 +1564,12 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
             msg.append('text2 is not a string (%s)'%(type(text2)))
         if msg:
             self.fail('\n'.join(msg))
-        self._difftext(text1.strip().splitlines(True), text2.strip().splitlines(True),
-                       junk,  msg_prefix)
+        lines1 = text1.strip().splitlines(True)
+        lines2 = text2.strip().splitlines(True)
+        if striplines:
+            lines1 = [line.strip() for line in lines1]
+            lines2 = [line.strip() for line in lines2]
+        self._difftext(lines1, lines2, junk,  msg_prefix)
     assertTextEqual = assertTextEquals
 
     def assertStreamEquals(self, stream1, stream2, junk=None,
@@ -1578,7 +1660,14 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
 
 
     def assertIsInstance(self, obj, klass, msg=None, strict=False):
-        """compares two files using difflib"""
+        """check if an object is an instance of a class
+
+        :param obj: the Python Object to be checked
+        :param klass: the target class
+        :param msg: a String for a custom message
+        :param strict: if True, check that the class of <obj> is <klass>;
+                       else check with 'isinstance'
+        """
         if msg is None:
             if strict:
                 msg = '%r is not of class %s but of %s'
@@ -1591,7 +1680,12 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
             self.assert_(isinstance(obj, klass), msg)
 
     def assertIs(self, obj, other, msg=None):
-        """compares identity of two reference"""
+        """compares identity of two reference
+
+        :param obj: a Python Object
+        :param other: another Python Object
+        :param msg: a String for a custom message
+        """
         if msg is None:
             msg = "%r is not %r"%(obj, other)
         self.assert_(obj is other, msg)
@@ -1604,7 +1698,10 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
         self.assert_(obj is not other, msg )
 
     def assertNone(self, obj, msg=None):
-        """assert obj is None"""
+        """assert obj is None
+
+        :param obj: Python Object to be tested
+        """
         if msg is None:
             msg = "reference to %r when None expected"%(obj,)
         self.assert_( obj is None, msg )
@@ -1616,7 +1713,14 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
         self.assert_( obj is not None, msg )
 
     def assertFloatAlmostEquals(self, obj, other, prec=1e-5, msg=None):
-        """compares two floats"""
+        """compares if two floats have a distance smaller than expected
+        precision.
+
+        :param obj: a Float
+        :param other: another Float to be comparted to <obj>
+        :param prec: a Float describing the precision
+        :param msg: a String for a custom message
+        """
         if msg is None:
             msg = "%r != %r" % (obj, other)
         self.assert_(math.fabs(obj - other) < prec, msg)
@@ -1631,6 +1735,11 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
         thrown, it will not be caught, and the test case will be
         deemed to have suffered an error, exactly as for an
         unexpected exception.
+
+        :param excClass: the Exception to be raised
+        :param callableObj: a callable Object which should raise <excClass>
+        :param args: a List of arguments for <callableObj>
+        :param kwargs: a List of keyword arguments  for <callableObj>
         """
         try:
             callableObj(*args, **kwargs)
@@ -1641,7 +1750,7 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
                 excName = excClass.__name__
             else:
                 excName = str(excClass)
-            raise self.failureException, "%s not raised" % excName
+            raise self.failureException("%s not raised" % excName)
 
     assertRaises = failUnlessRaises
 
@@ -1853,22 +1962,14 @@ class AttrObject: # XXX cf mock_object
     def __init__(self, **kwargs):
         self.__dict__.update(kwargs)
 
-def tag(*args):
+def tag(*args, **kwargs):
     """descriptor adding tag to a function"""
     def desc(func):
         assert not hasattr(func, 'tags')
-        func.tags = Tags(args)
+        func.tags = Tags(*args, **kwargs)
         return func
     return desc
 
-class Tags(set):
-    """A set of tag able validate an expression"""
-    def __getitem__(self, key):
-        return key in self
-
-    def match(self, exp):
-        return eval(exp, {}, self)
-
 def require_version(version):
     """ Compare version of python interpreter to the given one. Skip the test
     if older.
